@@ -4,11 +4,10 @@ package IkiWiki::Plugin::haiku;
 
 use warnings;
 use strict;
-use IkiWiki;
+use IkiWiki 2.00;
 
 sub import { #{{{
-	IkiWiki::hook(type => "preprocess", id => "haiku",
-		call => \&preprocess);
+	hook(type => "preprocess", id => "haiku", call => \&preprocess);
 } # }}}
 
 sub preprocess (@) { #{{{
@@ -16,7 +15,7 @@ sub preprocess (@) { #{{{
 
 	my $haiku;
 	eval q{use Coy};
-	if ($@) {
+	if ($@ || ! Coy->can("Coy::with_haiku")) {
 		my @canned=(
 			"The lack of a Coy:
 			 No darting, subtle haiku.
@@ -43,9 +42,9 @@ sub preprocess (@) { #{{{
 	}
 		
 	$haiku=~s/^\s+//mg;
-	$haiku=~s/\n/<br>\n/mg;
+	$haiku=~s/\n/<br \/>\n/mg;
 	
-	return "\n\n<blockquote>$haiku</blockquote>\n\n";
+	return "\n\n<blockquote><p>$haiku</p></blockquote>\n\n";
 } # }}}
 
 1
