@@ -4,7 +4,7 @@ package IkiWiki::Plugin::openid;
 
 use warnings;
 use strict;
-use IkiWiki;
+use IkiWiki 2.00;
 
 sub import { #{{{
 	hook(type => "getopt", id => "openid", call => \&getopt);
@@ -26,16 +26,26 @@ sub formbuilder_setup (@) { #{{{
 	my $form=$params{form};
 	my $session=$params{session};
 	my $cgi=$params{cgi};
-
+	
 	if ($form->title eq "signin") {
+		# Give up if module is unavailable to avoid
+		# needing to depend on it.
+		eval q{use Net::OpenID::Consumer};
+		if ($@) {
+			debug("unable to load Net::OpenID::Consumer, not enabling OpenID login");
+			return;
+		}
+
+		# This avoids it displaying a redundant label for the
+		# OpenID fieldset.
+		$form->fieldsets("OpenID");
+
  		$form->field(
 			name => "openid_url",
-			label => "OpenID",
+			label => gettext("Log in with")." ".htmllink("", "", "ikiwiki/OpenID", noimageinline => 1),
+			fieldset => "OpenID",
 			size => 30,
-			comment => '('.
-				htmllink("", "", "OpenID", 1, 0, "What's this?")
-				.($config{openidsignup} ? " | <a href=\"$config{openidsignup}\">Get an OpenID</a>" : "")
-				.')'
+			comment => ($config{openidsignup} ? " | <a href=\"$config{openidsignup}\">".gettext("Get an OpenID")."</a>" : "")
 		);
 
 		# Handle submission of an OpenID as validation.
@@ -58,8 +68,10 @@ sub formbuilder_setup (@) { #{{{
 	}
 	elsif ($form->title eq "preferences") {
 		if (! defined $form->field(name => "name")) {
-			$form->field(name => "OpenID", disabled => 1, value =>
-				$session->param("name"), size => 30, force => 1);
+			$form->field(name => "OpenID", disabled => 1,
+				value => $session->param("name"), 
+				size => 50, force => 1,
+				fieldset => "login");
 		}
 	}
 }
@@ -140,15 +152,43 @@ sub getobj ($$) { #{{{
 	# Store the secret in the session.
 	my $secret=$session->param("openid_secret");
 	if (! defined $secret) {
-		$secret=$session->param(openid_secret => time);
+		$secret=rand;
+		$session->param(openid_secret => $secret);
 	}
 
 	return Net::OpenID::Consumer->new(
 		ua => $ua,
 		args => $q,
-		consumer_secret => $secret,
+		consumer_secret => sub { return shift()+$secret },
 		required_root => $config{cgiurl},
 	);
 } #}}}
 
+package IkiWiki;
+
+# This is not used by this plugin, but this seems the best place to put it.
+# Used elsewhere to pretty-display the name of an openid user.
+sub openiduser ($) { #{{{
+	my $user=shift;
+
+	if ($user =~ m!^https?://! &&
+	    eval q{use Net::OpenID::VerifiedIdentity; 1} && !$@) {
+		my $oid=Net::OpenID::VerifiedIdentity->new(identity => $user);
+		my $display=$oid->display;
+		# Convert "user.somehost.com" to "user [somehost.com]".
+		if ($display !~ /\[/) {
+			$display=~s/^(.*?)\.([^.]+\.[a-z]+)$/$1 [$2]/;
+		}
+		# Convert "http://somehost.com/user" to "user [somehost.com]".
+		if ($display !~ /\[/) {
+			$display=~s/^https?:\/\/(.+)\/([^\/]+)$/$2 [$1]/;
+		}
+		$display=~s!^https?://!!; # make sure this is removed
+		eval q{use CGI 'escapeHTML'};
+		error($@) if $@;
+		return escapeHTML($display);
+	}
+	return;
+}
+
 1
