@@ -6,7 +6,7 @@ package IkiWiki::Plugin::sidebar;
 
 use warnings;
 use strict;
-use IkiWiki;
+use IkiWiki 2.00;
 
 sub import { #{{{
 	hook(type => "pagetemplate", id => "sidebar", call => \&pagetemplate);
@@ -27,10 +27,10 @@ sub sidebar_content ($) { #{{{
 
 		my $content=readfile(srcfile($sidebar_file));
 		return unless length $content;
-		return IkiWiki::htmlize($page, $sidebar_type,
+		return IkiWiki::htmlize($sidebar_page, $page, $sidebar_type,
 		       IkiWiki::linkify($sidebar_page, $page,
 		       IkiWiki::preprocess($sidebar_page, $page,
-		       IkiWiki::filter($sidebar_page, $content))));
+		       IkiWiki::filter($sidebar_page, $page, $content))));
 	}
 
 } # }}}
