@@ -3,13 +3,13 @@ package IkiWiki::Plugin::toggle;
 
 use warnings;
 use strict;
-use IkiWiki;
+use IkiWiki 2.00;
 
 # Here's the javascript that makes this possible. A key feature is the use
 # of css to hide toggleables, to avoid any flashing on page load. The css
 # is only emitted after the javascript tests that it's going to be able to
 # show the toggleables.
-my $javascript=<<'EOF';
+our $javascript=<<'EOF';
 <script type="text/javascript">
 <!--
 if (document.getElementById && document.getElementsByTagName && document.createTextNode) {
@@ -21,7 +21,8 @@ function inittoggle() {
 	var as = getElementsByClass('toggle');
 	for (var i = 0; i < as.length; i++) {
 		var id = as[i].href.match(/#(\w.+)/)[1];
-		document.getElementById(id).style.display="none";
+		if (document.getElementById(id).className == "toggleable")
+			document.getElementById(id).style.display="none";
 		as[i].onclick = function() {
 			toggle(this);
 			return false;
@@ -38,9 +39,9 @@ function toggle(s) {
 		style.display = "none";
 }
 
-function getElementsByClass(class) {
+function getElementsByClass(c) {
 	var ret = new Array();
-	var pattern = new RegExp("(^|\\s)"+class+"(\\s|$)");
+	var pattern = new RegExp("(^|\\s)"+c+"(\\s|$)");
 	var els = document.getElementsByTagName('*');
 	for (i = 0, j = 0; i < els.length; i++) {
 		if ( pattern.test(els[i].className) ) {
@@ -84,28 +85,33 @@ sub preprocess_toggle (@) { #{{{
 } # }}}
 
 sub preprocess_toggleable (@) { #{{{
-	my %params=(id => "default", text => "", @_);
+	my %params=(id => "default", text => "", open => "no", @_);
 
 	# Preprocess the text to expand any preprocessor directives
 	# embedded inside it.
-	$params{text}=IkiWiki::preprocess($params{page}, $params{destpage}, $params{text});
+	$params{text}=IkiWiki::preprocess($params{page}, $params{destpage}, 
+		IkiWiki::filter($params{page}, $params{destpage}, $params{text}));
 	
 	my $id=genid($params{page}, $params{id});
+	my $class=(lc($params{open}) ne "yes") ? "toggleable" : "toggleable-open";
 
 	# Should really be a postprocessor directive, oh well. Work around
 	# markdown's dislike of markdown inside a <div> with various funky
 	# whitespace.
 	my ($indent)=$params{text}=~/( +)$/;
 	$indent="" unless defined $indent;
-	return "<div class=\"toggleable\" id=\"$id\"></div>\n\n$params{text}\n$indent<div class=\"toggleableend\"></div>";
+	return "<div class=\"$class\" id=\"$id\"></div>\n\n$params{text}\n$indent<div class=\"toggleableend\"></div>";
 } # }}}
 
 sub format (@) { #{{{
         my %params=@_;
 
-	if ($params{content}=~s!(<div class="toggleable" id="[^"]+">)</div>!$1!g) {
+	if ($params{content}=~s!(<div class="toggleable(?:-open)?" id="[^"]+">\s*)</div>!$1!g) {
 		$params{content}=~s/<div class="toggleableend">//g;
-		$params{content}=~s!^<\/body>!$javascript</body>!m;
+		if (! ($params{content}=~s!^<body>!<body>$javascript!m)) {
+			# no </body> tag, probably in preview mode
+			$params{content}=$javascript.$params{content};
+		}
 	}
 	return $params{content};
 } # }}}
