@@ -55,7 +55,7 @@ sub mercurial_log($) {
 }
 
 sub rcs_update () { #{{{
-	my @cmdline = ("hg", "-R", "$config{srcdir}", "update");
+	my @cmdline = ("hg", "-q", "-R", "$config{srcdir}", "update");
 	if (system(@cmdline) != 0) {
 		warn "'@cmdline' failed: $!";
 	}
@@ -72,16 +72,19 @@ sub rcs_commit ($$$;$$) { #{{{
 		$user = possibly_foolish_untaint($user);
 	}
 	elsif (defined $ipaddr) {
-		$user = "Anonymous from $ipaddr";
+		$user = "Anonymous from ".possibly_foolish_untaint($ipaddr);
 	}
 	else {
 		$user = "Anonymous";
 	}
 
 	$message = possibly_foolish_untaint($message);
+	if (! length $message) {
+		$message = "no message given";
+	}
 
-	my @cmdline = ("hg", "-R", "$config{srcdir}", "commit", 
-	               "-m", "$message", "-u", "$user");
+	my @cmdline = ("hg", "-q", "-R", $config{srcdir}, "commit", 
+	               "-m", $message, "-u", $user);
 	if (system(@cmdline) != 0) {
 		warn "'@cmdline' failed: $!";
 	}
@@ -92,7 +95,7 @@ sub rcs_commit ($$$;$$) { #{{{
 sub rcs_add ($) { # {{{
 	my ($file) = @_;
 
-	my @cmdline = ("hg", "-R", "$config{srcdir}", "add", "$file");
+	my @cmdline = ("hg", "-q", "-R", "$config{srcdir}", "add", "$config{srcdir}/$file");
 	if (system(@cmdline) != 0) {
 		warn "'@cmdline' failed: $!";
 	}
@@ -101,10 +104,8 @@ sub rcs_add ($) { # {{{
 sub rcs_recentchanges ($) { #{{{
 	my ($num) = @_;
 
-	eval q{use CGI 'escapeHTML'};
-	error($@) if $@;
-
-	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", $num);
+	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", $num,
+		"--style", "default");
 	open (my $out, "@cmdline |");
 
 	eval q{use Date::Parse};
@@ -147,7 +148,7 @@ sub rcs_recentchanges ($) { #{{{
 	return @ret;
 } #}}}
 
-sub rcs_notify () { #{{{
+sub rcs_diff ($) { #{{{
 	# TODO
 } #}}}
 
@@ -156,7 +157,8 @@ sub rcs_getctime ($) { #{{{
 
 	# XXX filename passes through the shell here, should try to avoid
 	# that just in case
-	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", '1', $file);
+	my @cmdline = ("hg", "-R", $config{srcdir}, "log", "-v", "-l", '1', 
+		"--style", "default", "$config{srcdir}/$file");
 	open (my $out, "@cmdline |");
 
 	my @log = mercurial_log($out);
