@@ -24,25 +24,28 @@ sub setup_standard {
 		push @{$setup{plugin}}, @{$setup{add_plugins}};
 		delete $setup{add_plugins};
 	}
-	if (exists $setup{disable_plugins}) {
-		foreach my $plugin (@{$setup{disable_plugins}}) {
-			$setup{plugin}=[grep { $_ ne $plugin } @{$setup{plugin}}];
-		}
-		delete $setup{disable_plugins};
+	if (exists $setup{exclude}) {
+		push @{$config{wiki_file_prune_regexps}}, $setup{exclude};
 	}
 
-	debug("generating wrappers..");
-	my @wrappers=@{$setup{wrappers}};
-	delete $setup{wrappers};
-	my %startconfig=(%config);
-	foreach my $wrapper (@wrappers) {
-		%config=(%startconfig, verbose => 0, %setup, %{$wrapper});
-		checkconfig();
-		gen_wrapper();
+	if (! $config{render} && (! $config{refresh} || $config{wrappers})) {
+		debug(gettext("generating wrappers.."));
+		my @wrappers=@{$setup{wrappers}};
+		delete $setup{wrappers};
+		my %startconfig=(%config);
+		foreach my $wrapper (@wrappers) {
+			%config=(%startconfig, rebuild => 0, verbose => 0, %setup, %{$wrapper});
+			checkconfig();
+			if (! $config{cgi} && ! $config{post_commit}) {
+				$config{post_commit}=1;
+			}
+			gen_wrapper();
+		}
+		%config=(%startconfig);
 	}
-	%config=(%startconfig);
 	
 	foreach my $c (keys %setup) {
+		next if $c eq 'syslog';
 		if (defined $setup{$c}) {
 			if (! ref $setup{$c}) {
 				$config{$c}=possibly_foolish_untaint($setup{$c});
@@ -50,26 +53,40 @@ sub setup_standard {
 			elsif (ref $setup{$c} eq 'ARRAY') {
 				$config{$c}=[map { possibly_foolish_untaint($_) } @{$setup{$c}}]
 			}
+			elsif (ref $setup{$c} eq 'HASH') {
+				foreach my $key (keys %{$setup{$c}}) {
+					$config{$c}{$key}=possibly_foolish_untaint($setup{$c}{$key});
+				}
+			}
 		}
 		else {
 			$config{$c}=undef;
 		}
 	}
-
+	
 	if (! $config{refresh}) {
 		$config{rebuild}=1;
-		debug("rebuilding wiki..");
 	}
+	
+	loadplugins();
+	checkconfig();
+
+	if ($config{render}) {
+		commandline_render();
+	}
+
+	if (! $config{refresh}) {
+		debug(gettext("rebuilding wiki.."));
+	}
 	else {
-		debug("refreshing wiki..");
+		debug(gettext("refreshing wiki.."));
 	}
 
-	checkconfig();
 	lockwiki();
 	loadindex();
 	refresh();
 
-	debug("done");
+	debug(gettext("done"));
 	saveindex();
 }
 
