@@ -11,16 +11,23 @@ sub userinfo_retrieve () { #{{{
 	my $userinfo=eval{ Storable::lock_retrieve("$config{wikistatedir}/userdb") };
 	return $userinfo;
 } #}}}
-	
+
 sub userinfo_store ($) { #{{{
 	my $userinfo=shift;
 	
+	my $newfile="$config{wikistatedir}/userdb.new";
 	my $oldmask=umask(077);
-	my $ret=Storable::lock_store($userinfo, "$config{wikistatedir}/userdb");
+	my $ret=Storable::lock_nstore($userinfo, $newfile);
 	umask($oldmask);
+	if (defined $ret && $ret) {
+		if (! rename($newfile, "$config{wikistatedir}/userdb")) {
+			unlink($newfile);
+			$ret=undef;
+		}
+	}
 	return $ret;
 } #}}}
-	
+
 sub userinfo_get ($$) { #{{{
 	my $user=shift;
 	my $field=shift;
@@ -67,27 +74,22 @@ sub is_admin ($) { #{{{
 	return grep { $_ eq $user_name } @{$config{adminuser}};
 } #}}}
 
-sub commit_notify_list ($@) { #{{{
-	my $committer=shift;
-	
-	my @pages;
-	foreach my $file (@_) {
-		push @pages, grep { $pagesources{$_} eq $file } keys %pagesources;
-	}
-	
+sub get_banned_users () { #{{{
 	my @ret;
 	my $userinfo=userinfo_retrieve();
 	foreach my $user (keys %{$userinfo}) {
-		next if $user eq $committer;
-		if (exists $userinfo->{$user}->{subscriptions} &&
-		    length $userinfo->{$user}->{subscriptions} &&
-		    exists $userinfo->{$user}->{email} &&
-		    length $userinfo->{$user}->{email} &&
-		    grep { globlist_match($_, $userinfo->{$user}->{subscriptions}) } @pages) {
-			push @ret, $userinfo->{$user}->{email};
-		}
+		push @ret, $user if $userinfo->{$user}->{banned};
 	}
 	return @ret;
 } #}}}
 
+sub set_banned_users (@) { #{{{
+	my %banned=map { $_ => 1 } @_;
+	my $userinfo=userinfo_retrieve();
+	foreach my $user (keys %{$userinfo}) {
+		$userinfo->{$user}->{banned} = $banned{$user};
+	}
+	return userinfo_store($userinfo);
+} #}}}
+
 1
